/*
 * Copyright (c) 2006
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 *
 * Authors: Korey L. Sewell
 */

#ifndef __ARCH_MIPS_REGFILE_INT_REGFILE_HH__
#define __ARCH_MIPS_REGFILE_INT_REGFILE_HH__

#include "arch/mips/types.hh"
#include "arch/mips/isa_traits.hh"
#include "base/misc.hh"
#include "sim/faults.hh"

class Checkpoint;
class ThreadContext;

namespace MipsISA
{
    static inline std::string getIntRegName(RegIndex)
    {
        return "";
    }

    enum MiscIntRegNums {
       LO = NumIntArchRegs,
       HI,
       DSPACX0,
       DSPLo1,
       DSPHi1,
       DSPACX1,
       DSPLo2,
       DSPHi2,
       DSPACX2,
       DSPLo3,
       DSPHi3,
       DSPACX3,
       DSPControl,
       DSPLo0 = LO,
       DSPHi0 = HI
    };

    class IntRegFile
    {
      protected:
        IntReg regs[NumIntRegs];

      public:
        void clear() { bzero(&regs, sizeof(regs)); }

        IntReg readReg(int intReg)
        {
            return regs[intReg];
        }

        Fault setReg(int intReg, const IntReg &val)
        {
            if (intReg != ZeroReg) {
                regs[intReg] = val;
            }

            return NoFault;
        }

        void serialize(std::ostream &os);

        void unserialize(Checkpoint *cp, const std::string &section);

    };

} // namespace MipsISA

#endif
