/*
 * Copyright (c) 2006
 * The Regents of The University of Michigan
 * All Rights Reserved
 *
 * This code is part of the M5 simulator.
 *
 * Permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any
 * purpose, so long as the copyright notice above, this grant of
 * permission, and the disclaimer below appear in all copies made; and
 * so long as the name of The University of Michigan is not used in
 * any advertising or publicity pertaining to the use or distribution
 * of this software without specific, written prior authorization.
 *
 * THIS SOFTWARE IS PROVIDED AS IS, WITHOUT REPRESENTATION FROM THE
 * UNIVERSITY OF MICHIGAN AS TO ITS FITNESS FOR ANY PURPOSE, AND
 * WITHOUT WARRANTY BY THE UNIVERSITY OF MICHIGAN OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE. THE REGENTS OF THE UNIVERSITY OF MICHIGAN SHALL NOT BE
 * LIABLE FOR ANY DAMAGES, INCLUDING DIRECT, SPECIAL, INDIRECT,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, WITH RESPECT TO ANY CLAIM
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OF THE SOFTWARE, EVEN
 * IF IT HAS BEEN OR IS HEREAFTER ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGES.
 *
 * Authors: Gabe Black
 */

#ifndef __ARCH_MIPS_TLB_HH__
#define __ARCH_MIPS_TLB_HH__

#include "params/MipsDTB.hh"
#include "params/MipsITB.hh"
#include "sim/tlb.hh"

namespace MipsISA
{
    struct TlbEntry
    {
        Addr _pageStart;
        TlbEntry() {}
        TlbEntry(Addr asn, Addr vaddr, Addr paddr) : _pageStart(paddr) {}

        Addr pageStart()
        {
            return _pageStart;
        }

        void serialize(std::ostream &os);
        void unserialize(Checkpoint *cp, const std::string &section);
    };

    class TLB : public GenericTLB
    {
      public:
        typedef MipsTLBParams Params;
        TLB(const Params *p) : GenericTLB(p)
        {}

        Fault translate(RequestPtr req, ThreadContext *tc, bool=false);
    };

    class ITB : public TLB
    {
      public:
        typedef MipsITBParams Params;
        ITB(const Params *p) : TLB(p)
        {}
    };

    class DTB : public TLB
    {
      public:
        typedef MipsDTBParams Params;
        DTB(const Params *p) : TLB(p)
        {}
    };
};

#endif // __ARCH_MIPS_TLB_HH__
